@@ -1,99 +1,106 @@
 ## The BUGSdeclClass contains the pulled-apart content of a BUGS declaration line
 
 ## nimbleOrRfunctionNames is used to determine what (in BUGS code) can be evaluated in R if every argument is known OR in C++ (nimble) if arguments are other nodes
-nimbleOrRfunctionNames <- c('[',
-                            '+',
-                            '-',
-                            '/',
-                            '*',
-                            '(',
-                            'exp',
-                            'log',
-                            'pow',
-                            '^',
-                            '%%',
-                            '%*%',
-                            't',
-                            'equals',
-                            'inprod',
-                            'nimEquals',
-                            'sqrt',
-                            'logit',
-                            'expit',
-                            'ilogit',
-                            'probit',
-                            'iprobit',
-                            'phi',
-                            'cloglog',
-                            'icloglog',
-                            'step',
-                            'nimStep',
-                            'sin',
-                            'cos',
-                            'tan',
-                            'asin',
-                            'acos',
-                            'atan',
-                            'cosh',
-                            'sinh',
-                            'tanh',
-                            'asinh',
-                            'acosh',
-                            'atanh',
-                            'cube',
-                            'abs',
-                            'lgamma',
-                            'loggam',
-                            'log1p',
-                            'lfactorial',
-                            'besselK',
-                            'ceiling',
-                            'floor',
-                            'round',
-                            'nimRound',
-                            'trunc',
-                            'optim',
-                            'nimOptim',
-                            'optimDefaultControl',
-                            'nimOptimDefaultControl',
-                            'mean',
-                            'sum',
-                            'sd',
-                            'var',
-                            'max',
-                            'min',
-                            'pmin',
-                            'pmax',
-                            'prod',
-                            'asRow',
-                            'asCol',
-                            'logdet',    
-			    'chol',
-                            'inverse',
-                            'forwardsolve',
-                            'backsolve',
-                            'solve',
-                            'nimEigen',
-                            'nimSvd',  
-                            '>',
-                            '<',
-                            '>=',
-                            '<=',
-                            '==',
-                            '!=',
-                            '&',
-                            '|',
-                            '$',
-                            distributionFuns,
-                            # these are allowed in DSL as special
-                            # cases even though exp_nimble and
-                            # t_nonstandard are the canonical NIMBLE
-                            # distribution functions
-                            paste0(c('d','r','q','p'), 't'),
-                            paste0(c('d','r','q','p'), 'exp'),
-                            'nimC', 'nimRep', 'nimSeq', 'diag',
-                            'nimNumeric','nimMatrix','nimArray',
-                            'length')
+nimblePreevaluationFunctionNames <- c('+',
+                                      '-',
+                                      '/',
+                                      '*',
+                                      'exp',
+                                      'log',
+                                      'pow',
+                                      '^',
+                                      '%%',
+                                      'equals',
+                                      'nimEquals',
+                                      'sqrt',
+                                      'logit',
+                                      'expit',
+                                      'ilogit',
+                                      'probit',
+                                      'iprobit',
+                                      'phi',
+                                      'cloglog',
+                                      'icloglog',
+                                      'step',
+                                      'nimStep',
+                                      'sin',
+                                      'cos',
+                                      'tan',
+                                      'asin',
+                                      'acos',
+                                      'atan',
+                                      'cosh',
+                                      'sinh',
+                                      'tanh',
+                                      'asinh',
+                                      'acosh',
+                                      'atanh',
+                                      'cube',
+                                      'abs',
+                                      'lgamma',
+                                      'loggam',
+                                      'log1p',
+                                      'lfactorial',
+                                      'besselK',
+                                      'ceiling',
+                                      'floor',
+                                      'round',
+                                      'nimRound',
+                                      'trunc',
+                                      '>',
+                                      '<',
+                                      '>=',
+                                      '<=',
+                                      '==',
+                                      '!=',
+                                      '[',
+                                      '(',
+                                      '%*%',
+                                      't',
+                                      'inprod',
+                                      'optim',
+                                      'nimOptim',
+                                      'optimDefaultControl',
+                                      'nimOptimDefaultControl',
+                                      'mean',
+                                      'sum',
+                                      'sd',
+                                      'var',
+                                      'max',
+                                      'min',
+                                      'pmin',
+                                      'pmax',
+                                      'prod',
+                                      'asRow',
+                                      'asCol',
+                                      'logdet',    
+                                      'chol',
+                                      'inverse',
+                                      'forwardsolve',
+                                      'backsolve',
+                                      'solve',
+                                      'nimEigen',
+                                      'nimSvd',  
+                                      '&',
+                                      '|',
+                                      '$',
+                                      det_distributionFuns,
+                                        # these are allowed in DSL as special
+                                        # cases even though exp_nimble and
+                                        # t_nonstandard are the canonical NIMBLE
+                                        # distribution functions
+                                      paste0(c('d','q','p'), 't'),
+                                      paste0(c('d','q','p'), 'exp'),
+                                      'nimC', 'nimRep', 'nimSeq', 'diag',
+                                      'nimNumeric','nimMatrix','nimArray',
+                                      'length'
+                                      )
+
+nimbleOrRfunctionNames <- c(nimblePreevaluationFunctionNames,
+                            distribution_rFuns,
+                            paste0(c('r'), 't'),
+                            paste0(c('r'), 'exp')
+                            )
 
 functionsThatShouldNeverBeReplacedInBUGScode <- c(':','nimC','nimRep','nimSeq', 'diag',
                                                   'nimNumeric', 'nimMatrix', 'nimArray')
