@@ -80,9 +80,9 @@ cppNimbleListClass <- setRefClass('cppNimbleListClass',
                                           newCodeLine <- cppLiteral(c(paste0('ptrToSmartPtr = static_cast<nimSmartPtr<',name,'>* >(R_ExternalPtrAddr(input));'),
                                                                       'ptrToSmartPtrBase = dynamic_cast<nimSmartPtrBase*>(ptrToSmartPtr);',
                                                                       'ptrToPtr = ptrToSmartPtr->getVoidPtrToRealPtr();',
-                                                                      'SptrToSmartPtrBase = PROTECT(R_MakeExternalPtr(ptrToSmartPtrBase, R_NilValue, R_NilValue));',
-                                                                      'SptrToPtr = PROTECT(R_MakeExternalPtr(ptrToPtr, R_NilValue, R_NilValue));'))
-                                          allocVectorLine <- cppLiteral(paste0('Sans = PROTECT(Rf_allocVector(VECSXP,', 2, '));'))
+                                                                      'PROTECT(SptrToSmartPtrBase = R_MakeExternalPtr(ptrToSmartPtrBase, R_NilValue, R_NilValue));',
+                                                                      'PROTECT(SptrToPtr = R_MakeExternalPtr(ptrToPtr, R_NilValue, R_NilValue));'))
+                                          allocVectorLine <- cppLiteral(paste0('PROTECT(Sans = Rf_allocVector(VECSXP,', 2, '));'))
                                           
                                           packListLines <- cppLiteral(c('SET_VECTOR_ELT(Sans,0,SptrToSmartPtrBase);',
                                                                         'SET_VECTOR_ELT(Sans,1,SptrToPtr);'
@@ -132,17 +132,23 @@ cppNimbleListClass <- setRefClass('cppNimbleListClass',
 
                                           CBobjectDefs <- list(cppVarFull(name = 'ptrToSmartPtr', baseType = 'nimSmartPtr', templateArgs = name, ptr = 1),
                                                                cppVarFull(name = 'newObj', baseType = name, ptr = 1),
-                                                               SptrToSmartPtr = cppSEXP(name = 'SptrToSmartPtr'))
+                                                               SptrToSmartPtr = cppSEXP(name = 'SptrToSmartPtr'),
+                                                               Sans = cppSEXP(name = "Sans"))
                                           newCodeLine <- cppLiteral(c(paste('newObj = new ',name,';'),
                                                                       paste0('ptrToSmartPtr = new nimSmartPtr<',name,'>;'),
                                                                       'ptrToSmartPtr->setPtrFromT(newObj);',
-                                                                      'SptrToSmartPtr = PROTECT(R_MakeExternalPtr(ptrToSmartPtr, R_NilValue, R_NilValue));'))
-                                                                      
-                                          codeLines <- substitute({
-                                              ## Finalizer registration now happens through nimble's finalizer mapping system.
-                                              UNPROTECT(1)
-                                              return(ptrCastToPtrPairFun(SptrToSmartPtr))
-                                          }, list(ptrCastToPtrPairFun = as.name(ptrCastToPtrPairFun$name)))
+                                                                      'PROTECT(SptrToSmartPtr = R_MakeExternalPtr(ptrToSmartPtr, R_NilValue, R_NilValue));'))
+                                        # String pasting required to get the format PROTECT(Y = f(X)) instead of Y = PROTECT(f(X)).
+                                        # It might not matter, but we are being careful to make the PROTECT's look just like in Tomas Kalibera's blog post (and CRAN's code analysis)
+                                          codeLines <- cppLiteral(c(paste0('PROTECT(Sans = ', ptrCastToPtrPairFun$name, '(SptrToSmartPtr));'),
+                                                                    'UNPROTECT(2);',
+                                                                    'return(Sans);'))
+                                          ## codeLines <- substitute({
+                                          ##     ## Finalizer registration now happens through nimble's finalizer mapping system.
+                                          ##     Sans = PROTECT(ptrCastToPtrPairFun(SptrToSmartPtr))
+                                          ##     UNPROTECT(2)
+                                          ##     return(Sans)
+                                          ## }, list(ptrCastToPtrPairFun = as.name(ptrCastToPtrPairFun$name)))
                                           allCodeList <- list(newCodeLine, codeLines)
                                           allCode <- putCodeLinesInBrackets(allCodeList)
                                           SEXPgeneratorFun <<- cppFunctionDef(name = paste0('new_',name),
@@ -192,10 +198,10 @@ cppNimbleListClass <- setRefClass('cppNimbleListClass',
                                         listElementTable$addSymbol(cppSEXP(name = "S_newNimList"))
                                         listElementTable$addSymbol(cppSEXP(name = "S_listName"))
                                       
-                                        newListLine[[1]] <- substitute({S_listName <- PROTECT(Rf_allocVector(STRSXP, 1));
+                                        newListLine[[1]] <- substitute({PROTECT(S_listName <- Rf_allocVector(STRSXP, 1));
                                           SET_STRING_ELT(S_listName, 0, Rf_mkChar(LISTNAME));}, 
                                           list(LISTNAME = nimCompProc$nimbleListObj$className))
-                                        newListLine[[2]] <- substitute(S_newNimList <- PROTECT(makeNewNimbleList(S_listName)),
+                                        newListLine[[2]] <- substitute(PROTECT(S_newNimList <- makeNewNimbleList(S_listName)),
                                                                            list())
                                         newListLine[[3]] <- quote(cppLiteral('RObjectPointer = S_newNimList;'))
                                         newListLine[[4]] <-   substitute(UNPROTECT(2), list())
@@ -224,7 +230,7 @@ cppNimbleListClass <- setRefClass('cppNimbleListClass',
                                         conditionalClauseEnd <- list(quote(cppLiteral('}')))
                                         environmentCPPName <- Rname2CppName('S_.xData')  ## create SEXP for ref class environment 
                                         listElementTable$addSymbol(cppSEXP(name = environmentCPPName))
-                                        envLine <- substitute({ENVNAME <- PROTECT(Rf_allocVector(STRSXP, 1));
+                                        envLine <- substitute({PROTECT(ENVNAME <- Rf_allocVector(STRSXP, 1));
                                           SET_STRING_ELT(ENVNAME, 0, Rf_mkChar(".xData"));}, 
                                           list(ENVNAME = as.name(environmentCPPName)))
                                         
@@ -272,14 +278,14 @@ cppNimbleListClass <- setRefClass('cppNimbleListClass',
 
                                         environmentCPPName <- Rname2CppName('S_.xData')  ## create SEXP for ref class environment 
                                         listElementTable$addSymbol(cppSEXP(name = environmentCPPName))
-                                        envLine <- substitute({ENVNAME <- PROTECT(Rf_allocVector(STRSXP, 1));
+                                        envLine <- substitute({PROTECT(ENVNAME <- Rf_allocVector(STRSXP, 1));
                                           SET_STRING_ELT(ENVNAME, 0, Rf_mkChar(".xData"));}, 
                                           list(ENVNAME = as.name(environmentCPPName)))
 
                                         for(i in seq_along(argNames)) {
                                           Snames[i] <- Rname2CppName(paste0('S_', argNames[i]))
                                           listElementTable$addSymbol(cppSEXP(name = Snames[i]))
-                                            copyFromListLines[[i]] <- substitute(SVAR <- PROTECT(Rf_findVarInFrame(PROTECT(GET_SLOT(S_nimList_, XDATA)), Rf_install(ARGNAME))),
+                                            copyFromListLines[[i]] <- substitute(PROTECT(SVAR <- Rf_findVarInFrame(PROTECT(GET_SLOT(S_nimList_, XDATA)), Rf_install(ARGNAME))),
                                                                                list(ARGNAME = argNames[i], 
                                                                                     SVAR = as.name(Snames[i]),
                                                                                     XDATA = as.name(environmentCPPName)))
